import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "qreator"

MainView {
    objectName: "mainView"
    applicationName: "qreator"
    automaticOrientation: true

    headerColor: "#3a2c32"
    backgroundColor: "#875864"
    footerColor: "#9b616c"

    property bool debugMode: false
    property real pageSpacing: units.gu(6)
    id: root

    width: units.gu(80)
    height: units.gu(80)

    function debugLog(msg) {
        /* Display debug messages in the debug tab. Useful to present debug messages
           in a device

           @msg: string with the debug message to display
         */
        debugLogger.text += msg + "\n"
        console.log(msg)
    }

    Tabs {
        id: tabs

        Tab {
            objectName: "tabScanner"
            title: i18n.tr("Scan")
            page: Scanner {
                id: pageScanner
            }
        }

        Tab {
                objectName: "tabQreator"
                title: i18n.tr("Create")
                page: Creator {
                    id: pageCreator
                }
        }

        Tab {
            objectName: "TabDebug"
            title: i18n.tr("Debug")

            // FIXME: this property seems to have a weird effect: when set to
            // true, the tab contents are always visible, also when other tabs
            // are loaded
            //visible: root.debugMode? true : false

            page: Page {
                Column {
                    anchors.fill: parent

                    spacing: units.gu(1)

                    TextArea {
                        id: debugLogger
                        width: parent.width
                        height: parent.height - parent.spacing - buttonClear.height

                        readOnly: true
                        text: ""
                    }

                    Button {
                        id: buttonClear
                        text: "Clear"
                        width: parent.width

                        onClicked: debugLogger.text = "";
                    }
                }
            }
        }
    }
}
