import QtQuick 2.0
import Ubuntu.Components 0.1
import QZXing 1.3
import "../js/jsqrcode-combined.js" as QR

Item {

    // This property allows switching between decoding backends:
    // true: uses the JavaScript decoding backend
    // false: uses the C++ decoding backend
    property bool useJsbackend: false

    signal decodingSuccess(string tag)
    signal decodingStart()
    signal decodingFinish(bool success)

    function decode(imageUrl, offsetX, offsetY, width, heigth) {
        offsetX = (typeof offsetX === "undefined") ? 0: offsetX;
        offsetY = (typeof offsetY === "undefined") ? 0: offsetY;
        width = (typeof width === "undefined") ? 0: width;
        heigth = (typeof heigth === "undefined") ? 0: heigth;

        if (!useJsbackend) {
            decoder.decodeSubImageQML(imageUrl, offsetX, offsetY, width, heigth);
        } else {
            decodingcanvas.imageurl = imageUrl;
            decodingcanvas.loadImage(imageUrl);
        }
    }

    // For the C++ decoding backend
    QZXing {
        id: decoder
        onTagFound: {
            decodingSuccess(tag);
        }

        onDecodingStarted: {
            decodingStart();
        }

        onDecodingFinished: {
            decodingFinish(succeeded);
        }
    }

    // For the JavaScript decoding backend
    Canvas {
        id: decodingcanvas
        visible: false
        property string imageurl
        width: 320
        height: 240
        renderStrategy: Canvas.Immediate

        onImageLoaded: {
            debugLog("Loaded image:" + imageurl);
            var context = decodingcanvas.getContext("2d");
            context.drawImage(imageurl, 0, 0);

            // This fails with a warning:
            // "Pixel read back is not support in Cooperative mode, please try Theaded or Immediate mode" (sic)
            console.log("Data URL:", canvas.toDataURL());

            /* Let's deactivate decoding until resizing works
            QR.qrcode.imagedata = context.createImageData(imageurl);
            QR.qrcode.width = QR.qrcode.imagedata.width;
            QR.qrcode.height = QR.qrcode.imagedata.height;

            decodingStart();
            try {
                var result = QR.qrcode.process(context);
                decodingSuccess(result);
                decodingFinish(true);
                debugLog("Result: " + result);
            }
            catch(e) {
                debugLog("Could not decode: " + e );
                decodingFinish(false);
            }*/
        }
    }
}
