import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import QtMultimedia 5.0

Item {
    anchors.fill: parent

    /*
    Connections {
        target: Qt.application
        // FIXME: this paints the video output on top of the canvas
        onActiveChanged: (Qt.application.active) ? camera.start() : camera.stop();
    }*/

    Page {
            id: pagePreview

            onWidthChanged: canvas.requestPaint();
            onHeightChanged: canvas.requestPaint();

            tools: ToolbarItems {
                ToolbarButton {
                    action: Action {
                        text: i18n.tr("Decode")
                        iconSource: Qt.resolvedUrl("../img/camera.png")
                        onTriggered: captureImage();
                    }
                }
                ToolbarButton {
                    action: Action {
                        text: i18n.tr("Upload")
                        iconSource: Qt.resolvedUrl("../img/add.png")
                        onTriggered: PopupUtils.open(uploader);
                    }
                }
            }

            VideoOutput {
                id: videoPreview
                source: camera
                anchors {
                    top: parent.top
                    horizontalCenter: parent.horizontalCenter
                }
                //width: parent.width
                //height: parent.width

                focus: visible

                /* This rotation needs to be applied since the camera hardware in the
                   Galaxy Nexus phone is mounted at an angle inside the device, so the video
                   feed is rotated too.
                   FIXME: This should come from a system configuration option so that we
                   don't have to have a different codebase for each different device we want
                   to run on */
                orientation: device.naturalOrientation === "portrait"  ? -90 : 0
            }

            Canvas {
                id: canvas
                property string imageurl
                property real captureWidth: units.gu(40)
                property real captureHeight: units.gu(40)
                property real captureOriginX
                property real captureOriginY
                width: videoPreview.width
                height: videoPreview.height
                anchors.fill: videoPreview
                visible: true

                onPaint: {
                    var ctx = canvas.getContext('2d');

                    ctx.clearRect(0, 0, width, height);

                    ctx.fillStyle = "rgba(0, 0, 0, .5)";

                    console.log("Canvas width: " + width)
                    console.log("Canvas height: " + height)

                    captureOriginX = (width / 2) - (captureWidth / 2);
                    captureOriginY = (height / 2) - (captureHeight / 2);

                    ctx.fillRect(0, 0, width, height);

                    // Origin, a width, and a height
                    ctx.clearRect(captureOriginX, captureOriginY, captureWidth, captureHeight);
                }
            }
    }

    Decoder {
        id: decoder
        onDecodingSuccess: {
            debugLog("Tag found: " + tag);
            var resultTitle = i18n.tr("Your code contents");
            var resultMessage = tag;
            showResult(resultMessage, resultTitle, true);
        }

        onDecodingStart: {
            debugLog("Decoding started");
        }

        onDecodingFinish: {
            debugLog("Decoding finished " + (success==true ? "successfully" : "unsuccessfully"));
            if (!success) {
                // This dialog will go away as soon as we implement automatic scanning
                // But we might want to keep it for providing feedback when manually uploading
                // images to decode
                var resultTitle = i18n.tr("Oops!");
                var resultMessage = i18n.tr("The code's content could not be decoded");
                showResult(resultMessage, resultTitle, false);
            }
        }
    }

    Component {
        id: uploader

        UploadImageDialog {

            onAccepted: {
                if (inputText !== '') {
                    console.log("Uploaded image", inputText);
                    decoder.decode(inputText);
                } else {
                    console.log("Empty image path, ignored")
                }
            }

        }
    }

    Camera {
        id: camera
        flash.mode: Camera.FlashOff
        captureMode: Camera.CaptureStillImage
        focus.focusMode: Camera.FocusAuto
        exposure.exposureMode: Camera.ExposureAuto

        imageCapture {
            //resolution: Qt.size(640, 480)

            onImageSaved: {
                debugLog("Picture saved as " + path);
                decoder.decode(path, canvas.captureOriginX, canvas.captureOriginY,
                               canvas.captureWidth, canvas.captureHeight);
            }
        }
    }

    DeviceOrientation {
        id: device
    }

    function showResult(resultMessage, resultTitle, decodingSuccess) {
        /* Shows the decoding result in a dialog

           @resultMessage: Main message to display
           @resultTitle: Title of the dialog
           @decodingSuccess: True if the decoding was successful
        */

        PopupUtils.open(Qt.resolvedUrl("QrCodeDialog.qml"), pagePreview,
                        {
                            text: resultMessage,
                            title: resultTitle,
                            decodingSuccess: decodingSuccess
                         })
    }

    function captureImage(){
        /*
        FIXME: we need to decide whether to save the image at all on the
        file system (perhaps by intercepting onImageCaptured from the
        Camera), to save files individually or to always overwrite the same
        picture as a workaround (using captureToLocation)

        The later seems to work on the desktop, but it inexplicably crashes
        the camera when doing it on a device.

        Another option to complement it could be to delete the saved image from
        C++: QFile::remove(const QString &fileName);

        For now, we'll simply save all images
        */

        //var captureLocation = "/tmp/qrcode.jpg";
        //camera.imageCapture.captureToLocation(captureLocation);

        camera.imageCapture.capture()

    }
}
